#include "../../include/GPU/gpu.hpp"
#include <PSX/File/Processor/file_processor.hpp>
#include <PSX/Auxiliary/lz4_decompressor.hpp>
#include <PSX/GPU/gpu.hpp>
#include <stdio.h>

#ifdef JABYENGINE_PAL
    #include "splash_image_pal_boot.hpp"
#else
    #include "splash_image_ntsc_boot.hpp"
#endif //JABYENGINE_PAL

extern "C" uint32_t __boot_loader_end;

namespace JabyEngine {
    namespace GPU {
        static void decompress_logo() {
            LZ4Decompressor lz4_decomp(reinterpret_cast<uint8_t*>(&__boot_loader_end));

            switch(lz4_decomp.process(SplashScreen, sizeof(SplashScreen))) {
                case Progress::InProgress:
                    printf("Decompressing still in progress...\n");
                    break;

                case Progress::Error:
                    printf("Error decompressing!!!\n");
                    break;

                case Progress::Done:
                    printf("Done decompressing\n");
                    break;
            }
        }

        void display_logo() {
            decompress_logo();

            // Upload SplashScreen picture
            auto state = FileProcessor::create(&__boot_loader_end, SimpleTIM(32, 0, 0, 0));
            //                   v this will not be correct for now...
            while(state.process((sizeof(SplashScreen)/sizeof(uint32_t))) == Progress::InProgress);

            Display::enable();
        }

        void setup() {
            GP1.write(Command::GP1::Reset());
            Screen::configurate();
            Screen::exchange_buffer_and_display();
            
            GPU::wait_ready_for_CMD();
            quick_fill_fast(Color24::Black(), PositionU16(32, 0), SizeU16(Display::Width, Display::Height));
        }
    }
}